/*! @azure/msal-node v3.8.7 2026-02-10 */
'use strict';
/* eslint-disable header/header */
const name = "@azure/msal-node";
const version = "3.8.7";

export { name, version };
//# sourceMappingURL=packageMetadata.mjs.map
