import { AccountInfo, CredentialEntity } from "@azure/msal-common/node";
export declare function generateCredentialKey(credential: CredentialEntity): string;
export declare function generateAccountKey(account: AccountInfo): string;
//# sourceMappingURL=CacheHelpers.d.ts.map