/**
 * @internal
 * Checks if a given date string is in ISO 8601 format.
 *
 * @param dateString - The date string to be checked.
 * @returns boolean - Returns true if the date string is in ISO 8601 format, otherwise false.
 */
export declare function isIso8601(dateString: number | string): boolean;
//# sourceMappingURL=TimeUtils.d.ts.map